@@ -1,7 +1,3 @@
-#[cfg(test)]
-#[macro_use]
-extern crate float_cmp;
-
 #[cfg(test)]
 mod api;
 
